==> cl_appbase-0.0.35.tar.gz: lib/src/bloc/widgets/table_collection_builder.dart <==
import 'package:boost/boost.dart';
import 'package:cl_appbase/bloc.dart';
import 'package:cl_appbase/utils.dart';
import 'package:cl_appbase/widgets.dart';
import 'package:flutter/material.dart';

class TableCollectionBuilder<Item> extends StatefulWidget {
  final PagedCollectionController<Item> collection;
  final List<DataColumn> columns;
  final TableRowBuilder<Item> rowBuilder;
  final WidgetBuilder? loadingBuilder;
  final ErrorBuilder? errorBuilder;
  final List<int> pageSizeOptions;
  final double? columnSpacing;
  final int? sortColumnIndex;
  final bool sortAscending;
  final ValueSetter<bool?>? onSelectAll;
  final Decoration? decoration;
  final MaterialStateProperty<Color?>? dataRowColor;
  final double? dataRowHeight;
  final TextStyle? dataTextStyle;
  final MaterialStateProperty<Color?>? headingRowColor;
  final double? headingRowHeight;
  final TextStyle? headingTextStyle;
  final double? horizontalMargin;
  final bool showCheckboxColumn;
  final double? dividerThickness;
  final bool showBottomBorder;
  final double? checkboxHorizontalMargin;

  TableCollectionBuilder({
    Key? key,
    required this.collection,
    required this.columns,
    required this.rowBuilder,
    this.pageSizeOptions = const [10, 20, 50, 100],
    this.loadingBuilder,
    this.errorBuilder,
    this.columnSpacing,
    this.sortColumnIndex,
    this.sortAscending = true,
    this.onSelectAll,
    this.decoration,
    this.dataRowColor,
    this.dataRowHeight,
    this.dataTextStyle,
    this.headingRowColor,
    this.headingRowHeight,
    this.headingTextStyle,
    this.horizontalMargin,
    this.showCheckboxColumn = true,
    this.dividerThickness,
    this.showBottomBorder = false,
    this.checkboxHorizontalMargin,
  })  : assert(pageSizeOptions.isNotEmpty),
        super(key: key);

  @override
  _TableCollectionBuilderState<Item> createState() =>
      _TableCollectionBuilderState<Item>();
}

class _TableCollectionBuilderState<Item>
    extends State<TableCollectionBuilder<Item>> {
  final _continueSemaphore = Semaphore();
  int _page = 0;
  late int _pageSize;

  @override
  void initState() {
    super.initState();
    _pageSize = widget.pageSizeOptions.first;
    _loadPage(_page);
  }

  @override
  Widget build(BuildContext context) {
    return PropertyBuilder<PagedCollectionModel<Item>>(
      stream: widget.collection.stream,
      error: _buildError,
      value: _build,
      loading: _buildLoading,
    );
  }

  Widget _build(BuildContext context, PagedCollectionModel<Item> model) {
    return Column(
      crossAxisAlignment: CrossAxisAlignment.stretch,
      mainAxisSize: MainAxisSize.min,
      children: [
        _buildTable(context, model.items),
        PageNavigation(
          pageSize: _pageSize,
          pageSizeOptions: widget.pageSizeOptions,
          totalCount: model.size,
          currentPage: _page,
          onNextPagePressed: () => _loadPage(_page + 1),
          onPreviousPagePressed: () => _loadPage(_page - 1),
          onFirstPagePressed: () => _loadPage(0),
          onLastPagePressed: () =>
              _loadPage((model.size / _pageSize).ceil() - 1),
          onRowsPerPageChanged: (size) => _setPageSize(size, model.size),
        )
      ],
    );
  }

  Widget _buildLoading(BuildContext context) {
    return Column(
      crossAxisAlignment: CrossAxisAlignment.stretch,
      mainAxisSize: MainAxisSize.min,
      children: [
        _buildTable(context, []),
        if (widget.loadingBuilder != null) widget.loadingBuilder!(context),
        if (widget.loadingBuilder == null) LoadingSpinner()
      ],
    );
  }

  Widget _buildTable(BuildContext context, List<Item> items) {
    return DataTable(
      checkboxHorizontalMargin: widget.checkboxHorizontalMargin,
      columnSpacing: widget.columnSpacing,
      columns: widget.columns,
      dataTextStyle: widget.dataTextStyle,
      decoration: widget.decoration,
      dividerThickness: widget.dividerThickness,
      headingTextStyle: widget.headingTextStyle,
      onSelectAll: widget.onSelectAll,
      showCheckboxColumn: widget.showCheckboxColumn,
      dataRowHeight: widget.dataRowHeight,
      headingRowHeight: widget.headingRowHeight,
      horizontalMargin: widget.horizontalMargin,
      dataRowColor: widget.dataRowColor,
      headingRowColor: widget.headingRowColor,
      showBottomBorder: widget.showBottomBorder,
      sortAscending: widget.sortAscending,
      sortColumnIndex: widget.sortColumnIndex,
      rows: items
          .map((item) => widget.rowBuilder(context, item))
          .toList(growable: false),
    );
  }

  Widget _buildError(BuildContext context, dynamic error) {
    if (widget.loadingBuilder != null) {
      return widget.loadingBuilder!(context);
    } else {
      // fallback loading indicator
      return Text(error.toString()); //TODO errorText
    }
  }

  Future<void> _loadPage(int page, {bool refresh = false}) async {
    await _continueSemaphore.throttle(() async {
      _page = page;
      await widget.collection.loadPage(page, _pageSize, invalidate: refresh);
    });
  }

  void _setPageSize(int rows, int size) {
    setState(() {
      final pages = (size / rows).ceil();
      _pageSize = rows;
      _loadPage((_page > pages) ? pages - 1 : _page);
    });
  }
}

==> cl_appbase-0.0.46.tar.gz: lib/src/bloc/widgets/table_collection_builder.dart <==
import 'package:boost/boost.dart';
import 'package:cl_appbase/bloc.dart';
import 'package:cl_appbase/utils.dart';
import 'package:cl_appbase/widgets.dart';
import 'package:flutter/material.dart';

class TableCollectionBuilder<Item> extends StatefulWidget {
  final PagedCollectionController<Item> collection;
  final List<DataColumn> columns;
  final TableRowBuilder<Item> rowBuilder;
  final WidgetBuilder? loadingBuilder;
  final ErrorBuilder? errorBuilder;
  final List<int> pageSizeOptions;
  final double? columnSpacing;
  final int? sortColumnIndex;
  final bool sortAscending;
  final ValueSetter<bool?>? onSelectAll;
  final Decoration? decoration;
  final MaterialStateProperty<Color?>? dataRowColor;
  final double? dataRowHeight;
  final TextStyle? dataTextStyle;
  final MaterialStateProperty<Color?>? headingRowColor;
  final double? headingRowHeight;
  final TextStyle? headingTextStyle;
  final double? horizontalMargin;
  final bool showCheckboxColumn;
  final double? dividerThickness;
  final bool showBottomBorder;
  final double? checkboxHorizontalMargin;

  TableCollectionBuilder({
    Key? key,
    required this.collection,
    required this.columns,
    required this.rowBuilder,
    this.pageSizeOptions = const [10, 20, 50, 100],
    this.loadingBuilder,
    this.errorBuilder,
    this.columnSpacing,
    this.sortColumnIndex,
    this.sortAscending = true,
    this.onSelectAll,
    this.decoration,
    this.dataRowColor,
    this.dataRowHeight,
    this.dataTextStyle,
    this.headingRowColor,
    this.headingRowHeight,
    this.headingTextStyle,
    this.horizontalMargin,
    this.showCheckboxColumn = true,
    this.dividerThickness,
    this.showBottomBorder = false,
    this.checkboxHorizontalMargin,
  })  : assert(pageSizeOptions.isNotEmpty),
        super(key: key);

  @override
  _TableCollectionBuilderState<Item> createState() =>
      _TableCollectionBuilderState<Item>();
}

class _TableCollectionBuilderState<Item>
    extends State<TableCollectionBuilder<Item>> {
  final _continueSemaphore = Semaphore();
  int _page = 0;
  late int _pageSize;

  @override
  void initState() {
    super.initState();
    _pageSize = widget.pageSizeOptions.first;
    _loadPage(_page);
  }

  @override
  Widget build(BuildContext context) {
    return PropertyBuilder<PagedCollectionModel<Item>>(
      stream: widget.collection.stream,
      error: _buildError,
      value: _build,
      loading: _buildLoading,
    );
  }

  Widget _build(BuildContext context, PagedCollectionModel<Item> model) {
    return Column(
      crossAxisAlignment: CrossAxisAlignment.stretch,
      mainAxisSize: MainAxisSize.min,
      children: [
        _buildTable(context, model.items),
        PageNavigation(
          pageSize: _pageSize,
          pageSizeOptions: widget.pageSizeOptions,
          totalCount: model.size,
          currentPage: _page,
          onNextPagePressed: () => _loadPage(_page + 1),
          onPreviousPagePressed: () => _loadPage(_page - 1),
          onFirstPagePressed: () => _loadPage(0),
          onLastPagePressed: () =>
              _loadPage((model.size / _pageSize).ceil() - 1),
          onRowsPerPageChanged: (size) => _setPageSize(size, model.size),
        )
      ],
    );
  }

  Widget _buildLoading(BuildContext context) {
    return Column(
      crossAxisAlignment: CrossAxisAlignment.stretch,
      mainAxisSize: MainAxisSize.min,
      children: [
        _buildTable(context, []),
        if (widget.loadingBuilder != null) widget.loadingBuilder!(context),
        if (widget.loadingBuilder == null) LoadingSpinner()
      ],
    );
  }

  Widget _buildTable(BuildContext context, List<Item> items) {
    return DataTable(
      checkboxHorizontalMargin: widget.checkboxHorizontalMargin,
      columnSpacing: widget.columnSpacing,
      columns: widget.columns,
      dataTextStyle: widget.dataTextStyle,
      decoration: widget.decoration,
      dividerThickness: widget.dividerThickness,
      headingTextStyle: widget.headingTextStyle,
      onSelectAll: widget.onSelectAll,
      showCheckboxColumn: widget.showCheckboxColumn,
      dataRowHeight: widget.dataRowHeight,
      headingRowHeight: widget.headingRowHeight,
      horizontalMargin: widget.horizontalMargin,
      dataRowColor: widget.dataRowColor,
      headingRowColor: widget.headingRowColor,
      showBottomBorder: widget.showBottomBorder,
      sortAscending: widget.sortAscending,
      sortColumnIndex: widget.sortColumnIndex,
      rows: items
          .map((item) => widget.rowBuilder(context, item))
          .toList(growable: false),
    );
  }

  Widget _buildError(BuildContext context, dynamic error) {
    if (widget.loadingBuilder != null) {
      return widget.loadingBuilder!(context);
    } else {
      // fallback loading indicator
      return Text(error.toString()); //TODO errorText
    }
  }

  Future<void> _loadPage(int page, {bool refresh = false}) async {
    await _continueSemaphore.throttle(() async {
      _page = page;
      await widget.collection.loadPage(page, _pageSize, invalidate: refresh);
    });
  }

  void _setPageSize(int rows, int size) {
    setState(() {
      final pages = (size / rows).ceil();
      _pageSize = rows;
      _loadPage((_page >= pages) ? pages - 1 : _page);
    });
  }
}
